﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;

namespace Post12513
{
    public class MyShapeFileFeatureSource : ShapeFileFeatureSource
    {
        private double tolerance;

        public MyShapeFileFeatureSource()
            : base()
        { }

        public MyShapeFileFeatureSource(string shapefilePathName)
            : base(shapefilePathName)
        { }

        public MyShapeFileFeatureSource(string shapeFilePathName, string ShapeFilePathIndexFile)
            : base(shapeFilePathName, ShapeFilePathIndexFile)
        {
        }

        /// 
        /// Get a tolerance for filter the features for drawing whose size is small than it. 
        /// 
        public double Tolerance
        {
            get { return tolerance; }
            set { tolerance = value; }
        }

        public ShapeFileFeatureSource GetShapeFileFeatureSource()
        {
            return (this);
        }

        protected override Collection<string> GetFeatureIdsForDrawingCore(RectangleShape boundingBox, double screenWidth, double screenHeight)
        {
            Collection<string> featureIdsInBbox = base.GetFeatureIdsForDrawingCore(boundingBox, screenWidth, screenHeight);

            Collection<string> featureIdsForDrawing = new Collection<string>();

            // Filter the features that its drawn size is really small than a specified width/height.
            double resolutionX = boundingBox.Width / screenWidth;
            double resolutionY = boundingBox.Height / screenHeight;

            foreach (string id in featureIdsInBbox)
            {
                RectangleShape bbox = GetBoundingBoxById(id);

                // Calculate the width and height of the feature drawn on the map under current scale
                double drawingWidth = bbox.Width / resolutionX;
                double drawingHeight = bbox.Height / resolutionY;

                // just add the feature whose drawn size is bigger than sepcified tolerance.
                if (Math.Max(drawingWidth, drawingHeight) > Tolerance)
                {
                    featureIdsForDrawing.Add(id);
                }
            }

            return featureIdsForDrawing;
        }

        protected override Collection<Feature> GetFeaturesForDrawingCore(RectangleShape boundingBox, double screenWidth, double screenHeight, IEnumerable<string> returningColumnNames)
        {
            // Get all feature ids for drawing, it should be really fast.
            Collection<string> featureIdsForDrawing = GetFeatureIdsForDrawing(boundingBox, screenWidth, screenHeight);

            return base.GetFeaturesByIds(featureIdsForDrawing, returningColumnNames);
        }
    }
}
